<?php

namespace HCM\FPA\Entity;

use HCM\Util\Repositories;
use HCM\Util\DateUtil;
use HCM\Util\TkEntity;
use Zeedhi\Framework\ORM\DateTime;

class FpacalculofolhaEntity extends TkEntity
{

    private $itensCalculoFolha;
    
    public function __construct($entityManager, $hcmParameters) {
        $this->entityManager = $entityManager;
        $this->hcmParameters = $hcmParameters;
        
        $this->fpaCalculoFolha = Repositories::FPA_CALCULOFOLHA;
        parent::__construct(null,$entityManager);
    }
    
    public function retornaCalculoFolha(
        $nr_org, $tipo_movimento, $dt_ocorrencia, $nrTpModalidadeCalc, $nrOcorrenciaCalc
    ) {
        $calc_folha = $this->entityManager->getRepository(Repositories::FPA_CALCULOFOLHA)->retornaCalculoFolha(
            $nr_org, $tipo_movimento, $nrTpModalidadeCalc, $nrOcorrenciaCalc, $dt_ocorrencia, $dt_ocorrencia
        );

        if (!$calc_folha) {
            $this->inserePersisteCalculoFolha($nr_org, $tipo_movimento, $nrTpModalidadeCalc, $nrOcorrenciaCalc, $dt_ocorrencia);
            $calc_folha = $this->entityManager->getRepository(Repositories::FPA_CALCULOFOLHA)->retornaCalculoFolha(
                $nr_org, $tipo_movimento, $nrTpModalidadeCalc, $nrOcorrenciaCalc, $dt_ocorrencia, $dt_ocorrencia
            );
        }
        return $calc_folha;
    }


    public function carregaListaNrCalculoFolhaTodasOcorrencias(
        $nrOrg, $tipoMovimento, $dtOcorrencia, $nrTpModalidadeCalc
    ){
        $calculosFolha = $this->entityManager->getRepository(Repositories::FPA_CALCULOFOLHA)->retornaCalculoFolhaTodasOcorrencias(
            $nrOrg, $tipoMovimento, $nrTpModalidadeCalc, $dtOcorrencia, $dtOcorrencia
        );

        $listaNrCalculo = array();

        foreach ($calculosFolha as $calculo){
            $listaNrCalculo[] = $calculo->getNrcalculofolha();
        }

        return $listaNrCalculo;
    }

    public function inserePersisteCalculoFolha(
        $nr_org, $tipo_movimento, $nrTpModalidadeCalc, $nrOcorrenciaCalc,  $dt_ocorrencia
    ){
        $dt_ocorrencia = DateUtil::getDataDeString(substr($dt_ocorrencia, 0, 10), DateUtil::FORMATO_BRASILEIRO, true);

        $item = new $this->fpaCalculoFolha;
        $item->setNrcalculofolha(\HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'FPA_CALCULOFOLHA', 1, 12, true));
        $item->setNrorg($nr_org);
        $item->setDtocorrencia($dt_ocorrencia);
        $item->setNrtipomoviment($tipo_movimento);
        $item->setNrtpmodalidcal($nrTpModalidadeCalc);
        $item->setNrocorrecal($nrOcorrenciaCalc);
        
        $item->setDtinclusao(DateUtil::getDataAtual());
        $item->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
        $item->setCdoperinclusao($this->hcmParameters->getCdoperador());
        $item->setDtultatu(DateUtil::getDataAtual());
        $item->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
        $item->setCdoperultatu($this->hcmParameters->getCdoperador());
        $this->entityManager->persist($item);
        $this->entityManager->flush($item);
        
        $nrItemCalculo = $item->getNrcalculofolha();
        unset($item);
        
        return $nrItemCalculo;
    }

    public function retornaCalculoFolhaPeriodo(
        $nr_org, $nr_tipo_movimento, $dt_inicial, $dt_final, $nrTipoModalidadeCalc,
        $nrOcorrenciaCalc
    ){

        $lista='';
        $calc_folha = $this->entityManager->getRepository(Repositories::FPA_CALCULOFOLHA)->retornaCalculoFolha(
            $nr_org, $nr_tipo_movimento, $nrTipoModalidadeCalc, $nrOcorrenciaCalc, $dt_inicial, $dt_final
        );

        foreach ($calc_folha as $calc) {
            $lista = $lista.$calc->getNrcalculofolha().',';
        }

        $lista = substr($lista, 0, strlen($lista)-1);
        return $lista;
    }


    public function retornaListaCalculoFolhaPorPeriodo(
        $nrOrg, $tipoMovimento, $dtInicial, $dtFinal, $nrTpModalidadeCalc
    ){
        $calculosFolha = $this->entityManager->getRepository(Repositories::FPA_CALCULOFOLHA)->retornaCalculoFolhaTodasOcorrencias(
            $nrOrg, $tipoMovimento, $nrTpModalidadeCalc, $dtInicial, $dtFinal
        );

        $listaNrCalculo = array();

        foreach ($calculosFolha as $calculo){
            $listaNrCalculo[$calculo->getNrcalculofolha()] = $calculo;
        }

        return $listaNrCalculo;
    }

    public function insereCalculoFolha($nrorg, $dtocorrencia, $nrtipomoviment, $nrtpmodalidadecalc, $nrocorrencal) {
        $this->setNrorg($nrorg);
        $this->setDtocorrencia($dtocorrencia);
        $this->setNrtipomoviment($nrtipomoviment);
        $this->setNrtpmodalidcal($nrtpmodalidadecalc);
        $this->setNrocorrecal($nrocorrencal);
    }

    public function getItensCalculoFolha() {
        if (!$this->itensCalculoFolha) {
            $this->itensCalculoFolha = FpaitecalcfolhaEntity::getItensCalculoFolha(
                $this->getNrorg(), $this->getNrcalculofolha()
            );
        }
        return $this->itensCalculoFolha;
    }
    
    public function getCalculoFolha(
        $nrorg, $nrtipomovimento, $nrmodalidade, $nrocorrecalc, $dtocorrencia
    ){
        $calculoFolha = $this->entityManager->getRepository(Repositories::FPA_CALCULOFOLHA)->getCalculoFolha(
            $nrorg, $nrtipomovimento, $nrmodalidade, $nrocorrecalc, $dtocorrencia
        );

        return $calculoFolha;
    }
    
}

