<?php

namespace HCM\PTO\Controllers;

use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request\Row;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Request;

class ResumoSituacaoFrequenciaController extends Crud {

    protected $dataSourceSetResumoSituFrequencia = 'ptoResumoSituFreq_setResumoSituFrequencia';
    protected $dataSourceGetResumoSituFrequencia = 'resumoSituFrequencia';
    //protected $dataSourceSetResumoSituFrequencia = 'ptoResumoSituFreq_setResumoSituFrequencia';
    protected $dataSourceGetPeriodoApuracao      = 'ptoPeriodoApuracao_getPeriodoApuracao';
    protected $dataSourceGetMotivoOcorFreq       = 'ptoMotivoOcorFreq_getMotivoOcorFreq';
    protected $dataSourceGetSituFrequencia       = 'ptoSituFrequencia_getSituFrequencia';
    protected $dataSourceGetEventos              = 'fpaEventoM_getEventos';

    protected $parameterBagColumns = array('NRORG', 'NRORGPADRAO', 'DTMESCOMPETENC', 'DTINICIOMES', 'DTFIMMES', 'NRTIPOPERIODOAPURACAO');

    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag){
        $this->parameterBag = $parameterBag;
        parent::__construct($dataSourceManager);
    }

    public function setResumoSituFrequencia (DTO\Request\DataSet $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceSetResumoSituFrequencia;
        try{
            foreach($request->getDataSet()->getRows() as $row) {
                if($row['__is_new'] === true){
                    $row['DTINCLUSAO'] = $row['DTATUAL'];
                    $row['CDOPERINCLUSAO'] = $row['CDOPERLOG'];
                    $row['NRORGINCLUSAO'] = $row['NRORGTRAB'];
                    $row['DTULTATU'] = $row['DTATUAL'];
                    $row['CDOPERULTATU'] = $row['CDOPERLOG'];
                    $row['NRORGULTATU'] = $row['NRORGTRAB'];
                    $row['NRRESUMOSITUFREQ'] = \HCM\Util\NovoCodigo::geraCodigo( $row['NRORGTRAB'], 'PTO_RESUMOSITUFREQ', 1, 12, true);

                } else {
                    $row['DTULTATU'] = $row['DTATUAL'];
                    $row['CDOPERULTATU'] = $row['CDOPERLOG'];
                    $row['NRORGULTATU'] = $row['NRORGTRAB'];
                }

                $row['NRORG'] = $row['NRORGTRAB'];

                if(!array_key_exists('NRPROGOCORRENCIA', $row))
                    $row['NRPROGOCORRENCIA'] = null;

                if(!array_key_exists('NRMOTIVOOCORFREQ', $row))
                    $row['NRMOTIVOOCORFREQ'] = null;

                if(!array_key_exists('IDFORCASITUFREQ', $row))
                    $row['IDFORCASITUFREQ'] = null;

                if(!array_key_exists('QTREFAPUNOVASITUFREQ', $row))
                    $row['QTREFAPUNOVASITUFREQ'] = null;

                if(!array_key_exists('NRLOTELANCEVENTO', $row))
                    $row['NRLOTELANCEVENTO'] = null;

                if(!array_key_exists('IDATIVO', $row) || empty($row['IDATIVO']))
                    $row['IDATIVO'] = 'S';

                if(!array_key_exists('NRSITUFREQNOVA', $row))
                    $row['NRSITUFREQNOVA'] = null;

                if(!array_key_exists('NRSITUFREQUENCIA', $row))
                    $row['NRSITUFREQUENCIA'] = null;
                
                $row['QTREFAPURSITUFREQ'] = str_replace('.', '', $row['QTREFAPURSITUFREQ']);
                $row['QTREFAPURSITUFREQ'] = str_replace(',', '.', $row['QTREFAPURSITUFREQ']);
                
                $row['QTREFABONSITUFREQ'] = str_replace('.', '', $row['QTREFABONSITUFREQ']);
                $row['QTREFABONSITUFREQ'] = str_replace(',', '.', $row['QTREFABONSITUFREQ']);
            }
            parent::save($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function getResumoSituFrequencia (DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceGetResumoSituFrequencia);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData); 
        $response->addDataSet($dataSet);
    }    

    public function getPeriodoApuracao (DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceGetPeriodoApuracao);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }

    public function getMotivoOcorFreq (DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceGetMotivoOcorFreq);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData); 
        $response->addDataSet($dataSet);
    }

    public function getSituFrequencia (DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceGetSituFrequencia);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData); 
        $response->addDataSet($dataSet);
    }

    public function getEventos (DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceGetEventos);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData); 
        $response->addDataSet($dataSet);
    }

    public function deleteResumoSituFrequencia (DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            $this->dataSourceName = $this->dataSourceSetResumoSituFrequencia;
            parent::delete($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    protected function processConditionValue($condition) {
        return $condition['value'];
    }
    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }

        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );


        return $newFilterCriteria;
    }
}