<?php
namespace HCM\PTO\Services;

use HCM\PTO\Repository\GerarLoteEventosRepository;
use Doctrine\ORM\EntityManager;
use HCM\FPA\Services\LoteLancEventoService;   
use HCM\FPA\Services\ItemLoteLancService;
use HCM\Util\DateUtil;

class GerarLoteEventosService {

    /**
     * @var gerarLoteEventosRepository
     */
    protected $gerarLoteEventosRepository;
    protected $loteLancEventoService;

    public function __construct(EntityManager $entityManager, GerarLoteEventosRepository $gerarLoteEventosRepository, LoteLancEventoService $loteLancEventoService, ItemLoteLancService $itemLoteLancService) {
        $this->entityManager = $entityManager;
        $this->gerarLoteEventosRepository = $gerarLoteEventosRepository;
        $this->loteLancEventoService = $loteLancEventoService;
        $this->itemLoteLancService = $itemLoteLancService;
    }

    /**
     * @return int
     */    
    public function getResumoPorSelecao ($criterio, $inicio, $fim, $dtCompetencia, $nrTipoVinculo, $nrPeriodoApuracao, $dtPeriodoIni, $dtPeriodoFim, $nrOrgPadrao, $nrOrg, $lote = null) {
        $result = $this->gerarLoteEventosRepository->getResumoPorSelecao($criterio, $inicio, $fim, $dtCompetencia, $nrTipoVinculo, $nrPeriodoApuracao, $dtPeriodoIni, $dtPeriodoFim, $nrOrgPadrao, $nrOrg, $lote);
        return $result;
    }

    public function getResumoParaLote ($criterio, $inicio, $fim, $dtCompetencia, $nrTipoVinculo, $nrPeriodoApuracao, $dtPeriodoIni, $dtPeriodoFim, $nrOrgPadrao, $nrOrg) {
        $result = $this->gerarLoteEventosRepository->getResumoParaLote($criterio, $inicio, $fim, $dtCompetencia, $nrTipoVinculo, $nrPeriodoApuracao, $dtPeriodoIni, $dtPeriodoFim, $nrOrgPadrao, $nrOrg);
        return $result;
    }
    public function geraLote($criterio, $inicio, $fim, $dtCompetencia, $nrTipoVinculo, $nrPeriodoApuracao, $dtPeriodoIni, $dtPeriodoFim, $nrOrgPadrao, $nrOrg, $nmTipoVinculo) {
        set_time_limit(0);
        error_reporting(E_ERROR | E_PARSE);
        $msg = array();
        $resumos = $this->getResumoPorSelecao($criterio, $inicio, $fim, $dtCompetencia, $nrTipoVinculo, $nrPeriodoApuracao, $dtPeriodoIni, $dtPeriodoFim, $nrOrgPadrao, $nrOrg, 'S');
        $resumosLote = $this->getResumoParaLote($criterio, $inicio, $fim, $dtCompetencia, $nrTipoVinculo, $nrPeriodoApuracao, $dtPeriodoIni, $dtPeriodoFim, $nrOrgPadrao, $nrOrg);

        if(!empty($resumosLote)){
            foreach ($resumosLote as $item) {
                $disponibilidade = true;
                if (isset($nrTipoVinculo)) {
                    $bloq = $this->itemLoteLancService->verificaEventoBloqueado($nrOrg, $item['NREVENTOM'], $nrTipoVinculo);
                    if ($bloq !== null){
                        if ($bloq['NREVENTOMPARA'] === null) {
                            $this->msg[] = array('msg' => 'Operação bloqueada. O evento '.$item['NREVENTOM'].' não pode ser lançado para o vínculo "'.$item['NRVINCULOM'].' - '.$item['NMVINCULOM']. '"!');
                            $disponibilidade = false;
                        } else {
                            $msg[] = array('msg' => 'Operação bloqueada.  O evento '.$item['NREVENTOM'].' não pode ser lançado para o vínculo "'.$item['NRVINCULOM'].' - '.$item['NMVINCULOM'].'" e foi ajustado para o evento '.$bloq['NREVENTOMPARA'].'!');
                            $disponibilidade = true;
                            $item['NREVENTOM'] = $bloq['NREVENTOMPARA'];
                        }
                    } else {
                        $disponibilidade = true;
                    }
                }
                
                if ($disponibilidade === true) {
                    $nrsitufrequencia = '';
                    
                    $info = (is_null($item['QTREFAPURSITUFREQ'])) ? 0 : $item['QTREFAPURSITUFREQ'];
                    
                    $ref = $this->gerarLoteEventosRepository->getIdmediavalref($item['NREVENTOM'], $dtCompetencia);
                    
                    $info = DateUtil::getHoraStringDeFloat($info);
                    $info = DateUtil::getHoraFloatDeString($info);
                    if ($ref == 'REFERENCIA') {
                        $vrRefFolhaLote = $info;
                        $vrRefEvento = '';
                    } else {
                        $vrRefFolhaLote = '';
                        $vrRefEvento = $info;
                    }
                    
                    $itensLote[] = array(
                        'NRVINCULOM' => $item['NRVINCULOM'],
                        'NMVINCULOM' => '',
                        'NREVENTOM' => $item['NREVENTOM'],
                        'NMEVENTOH' => '',
                        'VRREFFOLHALOTE' => $vrRefFolhaLote,
                        'VREVENTOLOTE' => $vrRefEvento
                    );
                }
            }
            
            $this->loteLancEventoService->setOrigemLoteLanc('PTO');

            $dslotelanc = "IMPORTAÇAO DO RESUMO DE SITUAÇÃO DE FREQUÊNCIA ".$nmTipoVinculo. ' '. date('H:i:s');
            $origem = 'RESUMOSITUFREQ';

            $nrlote = $this->loteLancEventoService->geraLoteLancEvento($nrOrg, $dtCompetencia, $origem, $dslotelanc, 'PENDENTE', null, 1, $itensLote);
        } else {
            $nrlote = null;
        }

        try {
            $this->entityManager->getConnection()->beginTransaction();
            
            if(!empty($resumos)){                
                foreach ($resumos as $item) {
                    $resumoSituFreq = $this->entityManager->getRepository('\Folha\Entities\PtoResumositufreq')
                        ->findOneBy(array('nrresumositufreq' => $item['NRRESUMOSITUFREQ'], 'nrorg' => $item['NRORG']));
                    $resumoSituFreq->setNrlotelancevento($nrlote);
                    $resumoSituFreq->persist();
                }
            }

            $this->entityManager->flush();
            $this->entityManager->getConnection()->commit();
            
            $retorno = array($nrlote, $msg);
            return $retorno;
        } catch (\Exception $e) {
            $this->entityManager->getConnection()->rollBack();
            throw $e;
        }
    }

} 