<?php
/**
 * Created by PhpStorm.
 * User: matheussilva
 * Date: 28/10/14
 * Time: 14:06
 */

namespace HCM\Util;

use HCM\Util\DateUtil;
use Zeedhi\Framework\Session\Session;

define('APP_PATH', dirname(__FILE__) . "/");


class ManipulacaoExcel {
    
    const XLS   = '001';
    const XLSX  = '002';

    const IN  = '101';
    const OUT = '102';

    const CARACTER_ACENTUADO =     'ÀÁÂÃÄÅÆÇÈÉÊËÌÍÎÏÐÑÒÓÔÕÖØÙÚÛÜÝÞßàáâãäåæçèéêëìíîïðñòóôõöøùúûýýþÿ';
    const CARACTER_NAO_ACENTUADO = 'AAAAAAACEEEEIIIIDNOOOOOOUUUUYBSAAAAAAACEEEEIIIIDNOOOOOOUUUYYBY';
    const APENAS_CARACTERES_ALFANUMERICOS = '/[^a-zA-Z0-9\s]/';
    const CARACTER_BRANCO = ' ';
    const CARACTER_VAZIO = '';


    /**
     * Executa a criação do template e faz o download do mesmo
     * @param array $info
     * @param array $eventos
     * @param int $nrorg
     * @throws Exception
     * @throws \Exception
     */
    public function gravar($info, $eventos, $nrorg)
    {
        try {
            $file = $this->makeCode($info, $eventos);
            $this->download($file, $info[0]['ESTRUTGEREN'], $nrorg);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Insere a estrutura no arquivo
     * @param array $info
     * @param array $eventos
     * @throws \Exception
     */
    private function makeCode($info, $eventos)
    {
        try {
            $file = new \PHPExcel();
            $file->setActiveSheetIndex(0);
//            $file->getActiveSheet()->getStyle('A1:BA100000')->getFill()->applyFromArray(array('type' => \PHPExcel_Style_Fill::FILL_SOLID, 'startcolor' => array('rgb' => "FFFFFF")));
            $colLivre = $this->makeColHead($file);
            $this->makeColEventos($file, $colLivre, $eventos);
            $this->makeRowInfo($file, $info);

            return $file;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Adiciona os dados de cabeçalho do Excel
     *
     * @param \PHPExcel $file
     * @param String $nmestrutlegal
     * @param String $nmestrutgeren
     * @throws \Exception
     */
    private function makeHead($file, $nmestrutlegal, $nmestrutgeren)
    {
        try {
            $file->getActiveSheet()->setCellValue('B1', 'Estrutura Legal');
            $file->getActiveSheet()->getStyle('B1')->getBorders()->getOutline()->setBorderStyle(\PHPExcel_Style_Border::BORDER_THIN);
            $file->getActiveSheet()->setCellValue('C1', $nmestrutlegal);
            $file->getActiveSheet()->getStyle('C1')->getBorders()->getOutline()->setBorderStyle(\PHPExcel_Style_Border::BORDER_THIN);

            $file->getActiveSheet()->setCellValue('B2', 'Estrutura Gerencial');
            $file->getActiveSheet()->getStyle('B2')->getBorders()->getOutline()->setBorderStyle(\PHPExcel_Style_Border::BORDER_THIN);
            $file->getActiveSheet()->setCellValue('C2', $nmestrutgeren);
            $file->getActiveSheet()->getStyle('C2')->getBorders()->getOutline()->setBorderStyle(\PHPExcel_Style_Border::BORDER_THIN);

            $file->getActiveSheet()->setCellValue('B3', 'Competencia');
            $file->getActiveSheet()->getStyle('B3')->getBorders()->getOutline()->setBorderStyle(\PHPExcel_Style_Border::BORDER_THIN);
            $file->getActiveSheet()->setCellValue('C3', '##/##/####');
            $file->getActiveSheet()->getStyle('C3')->getBorders()->getOutline()->setBorderStyle(\PHPExcel_Style_Border::BORDER_THIN);

            $file->getActiveSheet()->getColumnDimension('B')->setAutoSize(true);
            $file->getActiveSheet()->getColumnDimension('C')->setAutoSize(true);

        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Insere o cabeçalho das colunas
     *
     * @param \PHPExcel $file
     *
     * @return String;
     *
     * @throws \Exception
     */
    private function makeColHead($file)
    {
        try {
            $colshead = array(
                'Vínc.',
                'Vínculo',
                'Situação Funcional',
                'Admissão',
                'Cargo',
                'Estrut. Gerencial');

            $colId = 'A';
            $rowId = 6;

            foreach ($colshead as $colhead) {
                $file->getActiveSheet()->setCellValue($colId . $rowId, $colhead);
                $file->getActiveSheet()->getStyle($colId . $rowId)->getBorders()->getOutline()->setBorderStyle(\PHPExcel_Style_Border::BORDER_THIN);
                $file->getActiveSheet()->getColumnDimension($colId++)->setAutoSize(true);
            }

            return $colId;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     *
     * @param \PHPExcel $file
     * @param String $colIniEventos
     * @param array $eventos
     * @throws \Exception
     */
    private function makeColEventos($file, $colIniEventos, $eventos)
    {
        try {
            $colId = $colIniEventos;
            $rowId = 5;
            $rowId2 = 6;

            foreach ($eventos as $evento) {
                $file->getActiveSheet()->setCellValue($colId . $rowId, $evento['CODIGO']);
                $file->getActiveSheet()->getStyle($colId . $rowId)->getBorders()->getOutline()->setBorderStyle(\PHPExcel_Style_Border::BORDER_THIN);
                $file->getActiveSheet()->setCellValue($colId . $rowId2, $evento['EVENTO']);
                $file->getActiveSheet()->getStyle($colId . $rowId2)->getBorders()->getOutline()->setBorderStyle(\PHPExcel_Style_Border::BORDER_THIN);
                $file->getActiveSheet()->getColumnDimension($colId)->setAutoSize(true);
                $colId++;
            }
            $file->getActiveSheet()->setCellValue($colId . $rowId2, 'Observação');
            $file->getActiveSheet()->getStyle($colId . $rowId2)->getBorders()->getOutline()->setBorderStyle(\PHPExcel_Style_Border::BORDER_THIN);
            $file->getActiveSheet()->getColumnDimension($colId )->setAutoSize(true);
            $colId++;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * @param \PHPExcel $file
     * @param array $infos
     * @throws \Exception
     */
    private function makeRowInfo( $file, $infos)
    {
        try {
            $rowId = 7;

            foreach ($infos as $info) {
                $colId = 'A';
                $file->getActiveSheet()->setCellValue($colId . $rowId, $info['ID']);
                $file->getActiveSheet()->getColumnDimension($colId)->setAutoSize(true);
                $colId++;
                $file->getActiveSheet()->setCellValue($colId . $rowId, $info['NOME']);
                $file->getActiveSheet()->getColumnDimension($colId)->setAutoSize(true);
                $colId++;
                $file->getActiveSheet()->setCellValue($colId . $rowId, $info['SITUFUNC']);
                $file->getActiveSheet()->getColumnDimension($colId)->setAutoSize(true);
                $colId++;
                $file->getActiveSheet()->setCellValue($colId . $rowId, $info['ADMISSAO']);
                $file->getActiveSheet()->getColumnDimension($colId)->setAutoSize(true);
                $colId++;
                $file->getActiveSheet()->setCellValue($colId . $rowId, $info['CARGO']);
                $file->getActiveSheet()->getColumnDimension($colId)->setAutoSize(true);
                $colId++;
                $file->getActiveSheet()->setCellValue($colId . $rowId, $info['NMGEREN']);
                $file->getActiveSheet()->getColumnDimension($colId)->setAutoSize(true);
                $rowId++;
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * @param String $string
     * @return mixed
     */
    private static function _removeCaracteresEspeciais($string)
    {
        $string = preg_replace(self::APENAS_CARACTERES_ALFANUMERICOS, self::CARACTER_VAZIO, $string);
        $string = str_replace('  ', ' ',$string);

        return $string;
    }

    /**
     * @param \PHPExcel $file
     * @param int $nrestrutgeren
     * @param int $nrorg
     * @throws Exception
     */
    private function download($file, $nrestrutgeren, $nrorg)
    {
        try {
            $nomeArquivo = "Formulario_";
            $nomeArquivo .= self::_removeCaracteresEspeciais($nrestrutgeren);
            $nomeArquivo .= '.xls';

            $expDir = APP_PATH . '/uploads/' . $nrorg . '/Exportacao/';
            if (!file_exists($expDir)) {
                mkdir($expDir, 0777, true);
            }
            $expDir = realpath($expDir);
            $write = \PHPExcel_IOFactory::createWriter($file, 'Excel5');
            $write->save($expDir . DIRECTORY_SEPARATOR . $nomeArquivo);

            $file = new \TkFile($expDir . DIRECTORY_SEPARATOR . $nomeArquivo);
            $file->download();
        } catch (\Exception $e) {
            throw $e;
        }

    }
    

    /********************************************* IMPORTAÇÃO *******************************************/

    /**
     * CSU010 - Preparação da informação.
     *
     * @param String  $caminho
     *
     * @return array
     * @throws \Exception
     */
    public function preparaInformacao( $caminho)
    {
        try {
            $file = \PHPExcel_IOFactory::load($caminho);
            $file->setActiveSheetIndex(0);

            $cabecalho = $this->preparaCabecalho($file);
            $eventos = $this->preparaEventos($file);
            $funcs = $this->preparaFunc($file);
            $info = $this->preparaInfo($file, $funcs, $eventos, $cabecalho);

            return array(
                'CABECALHO' => $cabecalho,
                'EVENTO' => $eventos,
                'FUNCIONARIO' => $funcs,
                'INFO' => $info
            );
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * CSU010a - Captura as informações do cabeçalho e retorna um array.
     *
     * @param \PHPExcel $file
     *
     * @return array
     * @throws \Exception
     */
    private function preparaCabecalho( $file)
    {
        try {
            $nmestrutlegal = $file->getActiveSheet()->getCell('C1')->getValue();
            $nmestrutgeren = $file->getActiveSheet()->getCell('C2')->getValue();

            $dtmescompetencia = $file->getActiveSheet()->getCell('C3')->getValue();
            $dtmescompetencia = DateUtil::getDataDeString(jdtojulian($dtmescompetencia + 2415032), DateUtil::FORMATO_AMERICANO_2, true);
//        $dtmescompetencia = $dtmescompetencia->format('d/m/Y');

            return array('NMESTRUTLEGAL' => $nmestrutlegal,
                'NMESTRUTGEREN' => $nmestrutgeren,
                'DTMESCOMPETENCIA' => $dtmescompetencia);
        } catch (\Exception $e) {
            throw $e;
        }

    }

    /**
     * CSU010b - Captura os eventos.
     *
     * @param \PHPExcel $file
     *
     * @return array
     * @throws \Exception
     */
    private function preparaEventos( $file)
    {
        try {$colId  = 'G';
            $rowId  = 5;
            $rowId2 = 6;
            $rowId3 = 7;
            $eventos = array();

            $nrevento = $file->getActiveSheet()->getCell($colId.$rowId)->getValue();
            $nmevento = $file->getActiveSheet()->getCell($colId++.$rowId2)->getValue();
            while(!empty($nrevento) && !empty($nmevento)){
                $eventos[] = array('CODIGO' => $nrevento, 'EVENTO' => $nmevento, 'COLUNA' => $colId);
                $nrevento = $file->getActiveSheet()->getCell($colId.$rowId)->getValue();
                $nmevento = $file->getActiveSheet()->getCell($colId++.$rowId2)->getValue();
                
            }
            
            //$this->preparaObservacao($colId, 7);

            return $eventos;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * CSU010c - Constrói o array com a informação dos eventos
     * @param \PHPExcel $file
     * @return array
     * @throws \Exception
     */
    private function preparaFunc( $file)
    {
        try {
            $colId = 'A';
            $colId2 = 'B';
            $colId3 = 'C';
            $colId4 = 'D';
            $colId5 = 'E';
            $rowId = 7;

            $funcs = array();

            $linha = $rowId;
            $idFunc = $file->getActiveSheet()->getCell($colId . $rowId)->getValue();
            $func = $file->getActiveSheet()->getCell($colId2 . $rowId)->getValue();
            $situFunc = $file->getActiveSheet()->getCell($colId3 . $rowId)->getValue();
            $admin = $file->getActiveSheet()->getCell($colId4 . $rowId)->getValue();
            $cargo = $file->getActiveSheet()->getCell($colId5 . $rowId++)->getValue();

            while (!empty($idFunc) || $idFunc == '0') {

                $funcs[] = array(
                    'CODIGO' => $idFunc,
                    'FUNCIONARIO' => $func,
                    'SITUFUNC' => $situFunc,
                    'ADMISSAO' => $admin,
                    'CARGO' => $cargo,
                    'LINHA' => $linha
                );

                $linha++;
                $idFunc = $file->getActiveSheet()->getCell($colId . $rowId)->getValue();
                $func = $file->getActiveSheet()->getCell($colId2 . $rowId)->getValue();
                $situFunc = $file->getActiveSheet()->getCell($colId3 . $rowId)->getValue();
                $admin = $file->getActiveSheet()->getCell($colId4 . $rowId)->getValue();
                $cargo = $file->getActiveSheet()->getCell($colId5 . $rowId++)->getValue();
            }
            return $funcs;
        } catch (\Exception $e) {
            throw $e;
        }
    }
    /**
     * CSU010d - Constrói o array com a informação dos eventos
     *
     * @param \PHPExcel $file
     * @param array $funcs
     * @param array $eventos
     * @param array $cabeca
     *
     * @return array
     * @throws \Exception
     */
    private function preparaInfo( $file, $funcs, $eventos, $cabeca)
    {
        try {
//          $colId      = 'F';
            $rowId = 7;
            $tamFuncs = count($funcs);
            $tamEventos = count($eventos);
            $colIdObs = 'G';
            for($i = 0; $i < $tamEventos; $i++){
                $colIdObs++;
            }

            $info = array();

            for ($c = 0; $c < $tamFuncs; $rowId++, $c++) {
                $colId = 'G';
                for ($d = 0; $d < $tamEventos; $colId++, $d++) {
                    $info[] = array(
                        'CODIGO' => $funcs[$c]['CODIGO'],
                        'DTMESCOMPETENCIA' => $cabeca['DTMESCOMPETENCIA'],
                        'EVENTO' => $eventos[$d]['CODIGO'],
                        'VALOR'  => $file->getActiveSheet()->getCell($colId.$rowId)->getValue(),
                        'OBSERVACAO' => $file->getActiveSheet()->getCell($colIdObs.$rowId)->getValue(),
                        'LINHA' => $rowId
                    );
                }
            }
            return $info;
        } catch (\Exception $e) {
            throw $e;
        }
    }

}